""" Middleware for handling path """

from starlette.types import ASGIApp, Receive, Scope, Send


class PathIdMiddleware:  # pylint: disable=too-few-public-methods
    """
    Update scope with path_id

    Needs to be positioned anywhere between the RoutingMiddleware
    and the ContextMiddleware to have access to the path_params
    provided by the RoutingMiddleware. Path_params are emptied
    during the response phase, so we cannot leverage
    the same logic as for logging and headers.
    """

    def __init__(self, app: ASGIApp) -> None:
        self.app = app

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        path_id: str = scope.get("path", "")
        if "path_params" in scope:
            for param_name, param_value in reversed(scope["path_params"].items()):
                endswith_param = path_id.endswith(param_value)
                pattern = "/" + str(param_value)
                substitute = f"/<{param_name}>"

                if not endswith_param:
                    # Parameter is somewhere in the url and not at the end
                    pattern += "/"
                    substitute += "/"

                # Substitute the right-most occurrence
                left, right = path_id.rsplit(pattern, 1)
                path_id = substitute.join((left, right))

        scope["state"]["path_id"] = path_id

        await self.app(scope, receive, send)



""" Custom middleware to handle exception with extra logging """

import logging


from connexion.exceptions import InternalServerError
from connexion.lifecycle import ConnexionResponse
from connexion.middleware.exceptions import ExceptionMiddleware

from starlette.requests import Request

from .utils import _logging_ctx_var

LOG = logging.getLogger(__name__)


class CustomExceptionMiddleware(
    ExceptionMiddleware  # type: ignore
):  # pylint: disable=too-few-public-methods
    """Custom Exception Middleware to handle unhandled exceptions"""

    @staticmethod
    def common_error_handler(_request: Request, exc: Exception) -> ConnexionResponse:
        """Default handler for any unhandled Exception"""
        LOG.error(
            {
                "message": "Unhandled exception occurred",
                "unhandled_exception": True,
            },
            exc_info=exc,
        )
        # this is needed as the unhandled exception are not
        # caught by the error handlers
        internal_server_error = InternalServerError()
        internal_server_error.ext = {"trace_id": _logging_ctx_var.get().get("trace_id")}

        return internal_server_error.to_problem()



import pytest

from unittest.mock import AsyncMock
from typing import Any

from asgimiddlewares import PathIdMiddleware


@pytest.mark.parametrize(
    ["scope", "result"],
    [
        pytest.param({"state": {}}, ""),
        pytest.param({"state": {}, "path": "v1/ping"}, "v1/ping"),
        pytest.param(
            {
                "state": {},
                "path": "v1/foo/aaa/bar/bbb",
                "path_params": {"paramA": "aaa", "paramB": "bbb"},
            },
            "v1/foo/<paramA>/bar/<paramB>",
        ),
        pytest.param(
            {
                "state": {},
                "path": "/v1/foo/foo_id/bar/bar_id",
                "path_params": {"foo": "foo_id", "bar": "bar_id"},
            },
            "/v1/foo/<foo>/bar/<bar>",
        ),
        pytest.param(
            {
                "state": {},
                "path": "/v1/foo/1/bar/1",
                "path_params": {"foo": "1", "bar": "1"},
            },
            "/v1/foo/<foo>/bar/<bar>",
        ),
        pytest.param(
            {
                "state": {},
                "path": "/v1/foo/foo/bar/bar",
                "path_params": {"foo": "foo", "bar": "bar"},
            },
            "/v1/foo/<foo>/bar/<bar>",
        ),
        pytest.param(
            {
                "state": {},
                "path": "/v1/foo/foo/bar/foo",
                "path_params": {"foo": "foo", "bar": "foo"},
            },
            "/v1/foo/<foo>/bar/<bar>",
        ),
    ],
)
@pytest.mark.asyncio
async def test_path_id_middleware(scope: dict[str, Any], result: str) -> None:
    mock_app = AsyncMock()
    middleware = PathIdMiddleware(mock_app)
    mock_receive = AsyncMock()
    mock_send = AsyncMock()

    await middleware(scope, mock_receive, mock_send)

    assert scope.get("state", {}).get("path_id") == result
